@@ -226,7 +226,8 @@ gtk_file_list_destroy(GtkObject *object)
   while(list){
     item = (GtkIconListItem *)list->data;
     file_item = (GtkFileListItem *)item->link;
-    g_free(file_item->file_name);
+    if(file_item->file_name) g_free(file_item->file_name);
+    file_item->file_name = NULL;
     g_free(item->link);
     item->link = NULL;
     list = list->next;
@@ -236,11 +237,13 @@ gtk_file_list_destroy(GtkObject *object)
   while(list){
     GtkFileListType *type;
     type = (GtkFileListType *)list->data;
-    g_free(type->extension);
+    if(type->extension) g_free(type->extension);
+    type->extension = NULL;
     file_list->types = g_list_remove_link(file_list->types, list);
     g_list_free_1(list);
     list = file_list->types;
   }
+  file_list->types = NULL;
 
   list = file_list->pixmaps;
   while(list){
@@ -249,10 +252,18 @@ gtk_file_list_destroy(GtkObject *object)
     g_list_free_1(list);
     list = file_list->pixmaps;
   }
+  file_list->pixmaps = NULL;
 
 
   g_free(GTK_FILE_LIST(object)->path); 
+  GTK_FILE_LIST(object)->path = NULL; 
+
   g_free(GTK_FILE_LIST(object)->filter); 
+  GTK_FILE_LIST(object)->filter = NULL; 
+
+  if (GTK_OBJECT_CLASS (parent_class)->destroy)
+  (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+
 }
 
 static void
@@ -297,9 +308,8 @@ gtk_file_list_init (GtkFileList *file_list)
 
   file_list->ntypes = 0;
 
-  for(i = 0; i <= GTK_FILE_LIST_CORE; i++){
+  for(i = 0; i <= GTK_FILE_LIST_CORE; i++)
     gtk_file_list_add_type(file_list, (const gchar **)pixmap_data[i]);
-  }
 
   types = default_types;
   while(types->extension){
@@ -442,9 +452,9 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
         } else {
            aux_path[length - 3] = '\0';
         }
-     } else if(strcmp(aux_path + length - 1, root) == 0 && length > 1){
+     } else if(strcmp(aux_path + length - 1, root) == 0){
        aux_path[length - 1] = '\0';
-     }  
+     }
 
      if(strlen(aux_path) == 0)
      {
@@ -586,9 +596,9 @@ gtk_file_list_open_dir(GtkFileList *file_list, const gchar *dir_path)
 
        gdk_pixmap_ref(icon->pixmap);
        gdk_bitmap_ref(icon->mask);
-       item = gtk_icon_list_add_from_pixmap(icon_list, 
-                                            icon->pixmap, 
-                                            icon->mask, 
+       item = gtk_icon_list_add_from_pixmap(icon_list,
+                                            icon->pixmap,
+                                            icon->mask,
                                             file_item->file_name, file_item);
 
        if(file_item->is_link){
@@ -689,13 +699,13 @@ gtk_file_list_add_type       (GtkFileList *file_list,
   GdkPixmap *pixmap;
   GdkBitmap *mask;
   gint type;
- 
-  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, 
+
+  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                                  gdk_colormap_get_system(),
                                                  &mask, NULL,
                                                  (gchar **)data);
 
-  type = gtk_file_list_add_type_with_pixmap(file_list, pixmap, mask); 
+  type = gtk_file_list_add_type_with_pixmap(file_list, pixmap, mask);
   gdk_pixmap_unref(pixmap);
   gdk_bitmap_unref(mask);
 
@@ -708,7 +718,7 @@ gtk_file_list_add_type_with_pixmap (GtkFileList *file_list,
 {
   GtkWidget *wpixmap;
   wpixmap = gtk_pixmap_new(pixmap, mask);
-  
+
   file_list->pixmaps = g_list_append(file_list->pixmaps, wpixmap);
   file_list->ntypes++;
   return (file_list->ntypes - 1);
