@@ -164,25 +164,25 @@ gtk_plot_dt_destroy(GtkObject *object)
 void 
 gtk_plot_dt_clear(GtkPlotDT *data)
 {
-  GTK_PLOT_DT_CLASS(GTK_OBJECT(data)->klass)->clear(data);
+  GTK_PLOT_DT_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(data)))->clear(data);
 }
 
 gboolean 
 gtk_plot_dt_add_node(GtkPlotDT *data, GtkPlotDTnode node)
 {
-  return(GTK_PLOT_DT_CLASS(GTK_OBJECT(data)->klass)->add_node(data, node));
+  return(GTK_PLOT_DT_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(data)))->add_node(data, node));
 }
 
 GtkPlotDTnode * 
 gtk_plot_dt_get_node(GtkPlotDT *data, gint idx)
 {
-  return(GTK_PLOT_DT_CLASS(GTK_OBJECT(data)->klass)->get_node(data, idx));
+  return(GTK_PLOT_DT_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(data)))->get_node(data, idx));
 }
 
 gboolean 
 gtk_plot_dt_triangulate(GtkPlotDT *data)
 {
-  return(GTK_PLOT_DT_CLASS(GTK_OBJECT(data)->klass)->triangulate(data));
+  return(GTK_PLOT_DT_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(data)))->triangulate(data));
 }
 
 static void 
