@@ -28,14 +28,18 @@
 #include "gtkplotsegment.h"
 #include "gtkpsfont.h"
 
+#define P_(string) string
+
 static void gtk_plot_segment_class_init 	(GtkPlotSegmentClass *klass);
 static void gtk_plot_segment_init 		(GtkPlotSegment *data);
-static void gtk_plot_segment_set_arg       (GtkObject *object,
-                                         GtkArg    *arg,
-                                         guint      arg_id);
-static void gtk_plot_segment_get_arg       (GtkObject *object,
-                                         GtkArg    *arg,
-                                         guint      arg_id);
+static void gtk_plot_segment_get_property         (GObject      *object,
+                                                 guint            prop_id,
+                                                 GValue          *value,
+                                                 GParamSpec      *pspec);
+static void gtk_plot_segment_set_property         (GObject      *object,
+                                                 guint            prop_id,
+                                                 const GValue          *value,
+                                                 GParamSpec      *pspec);
 
 static void gtk_plot_segment_draw_legend	(GtkPlotData *data, 
 					 gint x, gint y);
@@ -97,6 +101,7 @@ gtk_plot_segment_class_init (GtkPlotSegmentClass *klass)
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkPlotDataClass *data_class;
+  GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
   parent_class = gtk_type_class (gtk_plot_data_get_type ());
 
@@ -104,33 +109,51 @@ gtk_plot_segment_class_init (GtkPlotSegmentClass *klass)
   widget_class = (GtkWidgetClass *) klass;
   data_class = (GtkPlotDataClass *) klass;
 
-  object_class->set_arg = gtk_plot_segment_set_arg;
-  object_class->get_arg = gtk_plot_segment_get_arg;
+  gobject_class->set_property = gtk_plot_segment_set_property;
+  gobject_class->get_property = gtk_plot_segment_get_property;
 
-  gtk_object_add_arg_type ("GtkPlotSegment::arrow_mask",
-                           GTK_TYPE_ENUM,
-                           GTK_ARG_READWRITE,
-                           ARG_MASK);
-  gtk_object_add_arg_type ("GtkPlotSegment::relative",
-                           GTK_TYPE_BOOL,
-                           GTK_ARG_READWRITE,
-                           ARG_RELATIVE);
-  gtk_object_add_arg_type ("GtkPlotSegment::centered",
-                           GTK_TYPE_BOOL,
-                           GTK_ARG_READWRITE,
-                           ARG_CENTERED);
-  gtk_object_add_arg_type ("GtkPlotSegment::style",
-                           GTK_TYPE_UINT,
-                           GTK_ARG_READWRITE,
-                           ARG_STYLE);
-  gtk_object_add_arg_type ("GtkPlotSegment::width",
-                           GTK_TYPE_UINT,
-                           GTK_ARG_READWRITE,
-                           ARG_WIDTH);
-  gtk_object_add_arg_type ("GtkPlotSegment::length",
-                           GTK_TYPE_UINT,
-                           GTK_ARG_READWRITE,
-                           ARG_LENGTH);
+  g_object_class_install_property (gobject_class,
+                           ARG_MASK,
+  g_param_spec_int ("arrow_mask",
+                           P_(""),
+                           P_(""),
+                           0,G_MAXINT,0,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                           ARG_RELATIVE,
+  g_param_spec_boolean ("relative",
+                           P_(""),
+                           P_(""),
+                           FALSE,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                           ARG_CENTERED,
+  g_param_spec_boolean ("centered",
+                           P_(""),
+                           P_(""),
+                           FALSE,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                           ARG_STYLE,
+  g_param_spec_int ("style",
+                           P_(""),
+                           P_(""),
+                           0,G_MAXINT,0,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                           ARG_WIDTH,
+  g_param_spec_int ("width",
+                           P_(""),
+                           P_(""),
+                           0,G_MAXINT,0,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class,
+                           ARG_LENGTH,
+  g_param_spec_int ("length",
+                           P_(""),
+                           P_(""),
+                           0,G_MAXINT,0,
+                           G_PARAM_READABLE|G_PARAM_WRITABLE));
 
   data_class->draw_legend = gtk_plot_segment_draw_legend;
   data_class->draw_symbol = gtk_plot_segment_draw_symbol;
@@ -191,68 +214,70 @@ gtk_plot_segment_init (GtkPlotSegment *dataset)
 }
 
 static void
-gtk_plot_segment_set_arg (GtkObject      *object,
-                      GtkArg         *arg,
-                      guint           arg_id)
+gtk_plot_segment_set_property (GObject      *object,
+                             guint            prop_id,
+                             const GValue          *value,
+                             GParamSpec      *pspec)
 {
   GtkPlotSegment *data;
 
   data = GTK_PLOT_SEGMENT (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
       case ARG_MASK:
-        data->arrow_mask  = GTK_VALUE_ENUM(*arg);
+        data->arrow_mask  = g_value_get_int(value);
         break;
       case ARG_RELATIVE:
-        data->relative  = GTK_VALUE_BOOL(*arg);
+        data->relative  = g_value_get_boolean(value);
         break;
       case ARG_CENTERED:
-        data->centered  = GTK_VALUE_BOOL(*arg);
+        data->centered  = g_value_get_boolean(value);
         break;
       case ARG_WIDTH:
-        data->arrow_width  = GTK_VALUE_UINT(*arg);
+        data->arrow_width  = g_value_get_int(value);
         break;
       case ARG_LENGTH:
-        data->arrow_length  = GTK_VALUE_UINT(*arg);
+        data->arrow_length  = g_value_get_int(value);
         break;
       case ARG_STYLE:
-        data->arrow_style  = GTK_VALUE_UINT(*arg);
+        data->arrow_style  = g_value_get_int(value);
         break;
     }
 }
 
 static void
-gtk_plot_segment_get_arg (GtkObject      *object,
-                      GtkArg         *arg,
-                      guint           arg_id)
+gtk_plot_segment_get_property (GObject      *object,
+                             guint            prop_id,
+                             GValue          *value,
+                             GParamSpec      *pspec)
 {
   GtkPlotSegment *data;
 
   data = GTK_PLOT_SEGMENT (object);
 
-  switch (arg_id)
+  switch (prop_id)
     {
       case ARG_MASK:
-        GTK_VALUE_ENUM(*arg) = data->arrow_mask;
+        g_value_set_int(value, data->arrow_mask);
         break;
       case ARG_RELATIVE:
-        GTK_VALUE_BOOL(*arg) = data->relative;
+        g_value_set_boolean(value, data->relative);
         break;
       case ARG_CENTERED:
-        GTK_VALUE_BOOL(*arg) = data->centered;
+        g_value_set_boolean(value, data->centered);
         break;
       case ARG_WIDTH:
-        GTK_VALUE_UINT(*arg) = data->arrow_width;
+        g_value_set_int(value, data->arrow_width);
         break;
       case ARG_LENGTH:
-        GTK_VALUE_UINT(*arg) = data->arrow_length;
+        g_value_set_int(value, data->arrow_length);
         break;
       case ARG_STYLE:
-        GTK_VALUE_UINT(*arg) = data->arrow_style;
+        g_value_set_int(value, data->arrow_style);
         break;
       default:
-        arg->type = GTK_TYPE_INVALID;
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
     }
 }
