@@ -77,7 +77,7 @@ update(gpointer data)
  if(x > xmax){
    gtk_plot_set_xrange(plot, xmin + 5. , xmax + 5.);
    gtk_plot_canvas_paint(GTK_PLOT_CANVAS(canvas));
-   gtk_plot_canvas_refresh(GTK_PLOT_CANVAS(canvas));
+   gtk_widget_queue_draw(GTK_WIDGET(canvas));
  } else {
    gtk_plot_data_draw_points(dataset, 1);
    gtk_plot_refresh(plot, NULL);
